@@ -1,11 +1,13 @@
-// https://docs.microsoft.com/en-us/typography/opentype/spec/gvar
+//! A [Glyph Variations Table](
+//! https://docs.microsoft.com/en-us/typography/opentype/spec/gvar) implementation.
+
 // https://docs.microsoft.com/en-us/typography/opentype/spec/otvarcommonformats#tuple-variation-store
 
 use core::cmp;
 use core::convert::TryFrom;
 use core::num::NonZeroU16;
 
-use crate::{loca, GlyphId, OutlineBuilder, Rect, BBox, NormalizedCoordinate};
+use crate::{GlyphId, OutlineBuilder, Rect, BBox, NormalizedCoordinate};
 use crate::parser::{Stream, Offset, Offset16, Offset32, LazyArray16, F2DOT14};
 use crate::glyf::{self, Transform};
 
@@ -22,6 +24,8 @@ enum GlyphVariationDataOffsets<'a> {
     Long(LazyArray16<'a, Offset32>),
 }
 
+/// A [Glyph Variations Table](
+/// https://docs.microsoft.com/en-us/typography/opentype/spec/gvar).
 #[derive(Clone, Copy)]
 pub struct Table<'a> {
     axis_count: NonZeroU16,
@@ -31,7 +35,7 @@ pub struct Table<'a> {
 }
 
 impl<'a> Table<'a> {
-    // https://docs.microsoft.com/en-us/typography/opentype/spec/gvar#gvar-header
+    /// Parses a table from raw data.
     pub fn parse(data: &'a [u8]) -> Option<Self> {
         let mut s = Stream::new(data);
         let version: u32 = s.read()?;
@@ -108,30 +112,30 @@ impl<'a> Table<'a> {
         let data = self.glyphs_variation_data.get(start..end)?;
         parse_variation_data(coordinates, &self.shared_tuple_records, points_len, data, tuples)
     }
+
+    /// Outlines a glyph.
+    pub fn outline(
+        &self,
+        glyf_table: glyf::Table,
+        coordinates: &[NormalizedCoordinate],
+        glyph_id: GlyphId,
+        builder: &mut dyn OutlineBuilder,
+    ) -> Option<Rect> {
+        let mut b = glyf::Builder::new(Transform::default(), BBox::new(), builder);
+        let glyph_data = glyf_table.get(glyph_id)?;
+        outline_var_impl(glyf_table, self, glyph_id, glyph_data, coordinates, 0, &mut b);
+        b.bbox.to_rect()
+    }
 }
 
-
-pub(crate) fn outline(
-    loca_table: loca::Table,
-    glyf_table: &[u8],
-    gvar_table: &Table,
-    coordinates: &[NormalizedCoordinate],
-    glyph_id: GlyphId,
-    builder: &mut dyn OutlineBuilder,
-) -> Option<Rect> {
-    let mut b = glyf::Builder::new(Transform::default(), BBox::new(), builder);
-
-    let range = loca_table.glyph_range(glyph_id)?;
-    let glyph_data = glyf_table.get(range)?;
-
-    outline_var_impl(loca_table, glyf_table, gvar_table,
-                     glyph_id, glyph_data, coordinates, 0, &mut b);
-    b.bbox.to_rect()
+impl core::fmt::Debug for Table<'_> {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
+        write!(f, "Table {{ ... }}")
+    }
 }
 
 fn outline_var_impl<'a>(
-    loca_table: loca::Table,
-    glyf_table: &[u8],
+    glyf_table: glyf::Table,
     gvar_table: &Table,
     glyph_id: GlyphId,
     data: &[u8],
@@ -204,10 +208,9 @@ fn outline_var_impl<'a>(
             transform = Transform::combine(transform, component.transform);
 
             let mut b = glyf::Builder::new(transform, builder.bbox, builder.builder);
-            let range = loca_table.glyph_range(component.glyph_id)?;
-            let glyph_data = glyf_table.get(range)?;
+            let glyph_data = glyf_table.get(component.glyph_id)?;
             outline_var_impl(
-                loca_table, glyf_table, gvar_table, component.glyph_id,
+                glyf_table, gvar_table, component.glyph_id,
                 glyph_data, coordinates, depth + 1, &mut b,
             )?;
 
